# QuestBrowser class

# Copyright (C) 2015 Kieran Colford
#
# This file is part of UWaterloo-AddCourse.
#
# UWaterloo-AddCourse is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# UWaterloo-AddCourse is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with UWaterloo-AddCourse.  If not, see
# <http://www.gnu.org/licenses/>.


"""The QuestBrowser class that manages interactions with QUEST."""

import urllib
import urllib2
import bs4
from .error import *


class QuestBrowser:

    """An opener for browsing QUEST."""

    req = None

    def __init__(self, questid, password):
        self.opener = urllib2.build_opener()
        self.opener.add_handler(urllib2.HTTPSHandler())
        self.opener.add_handler(urllib2.HTTPRedirectHandler())
        self.opener.add_handler(urllib2.HTTPCookieProcessor())

        self.questid = questid.upper()

        self.login(password)

    def open(self, req):
        """Open a request req to QUEST via self.opener."""

        if isinstance(req.data, str):
            req.data = req.data.encode('ascii')
        return self.opener.open(req)

    def add_data(self, data):
        """Add data to self.req."""

        if self.req is None:
            raise Error("request cannot be None")
        self.req.data = data

    def get_page(self, req=None):
        """Return a BeautifulSoup instance for url."""

        if req is None:
            if self.req is None:
                raise Error("request cannot be None")
            req = self.req
            if self.data:
                req.data = urllib.urlencode(self.data)
                self.data = None

        f = self.open(req)
        self.page = f.read()
        f.close()

        if req is self.req:
            self.req = None

        self.page = bs4.BeautifulSoup(self.page)

        return self.page

    def make_request(self, page, query={}):
        """Make a Request object for a QUEST page."""

        self.data = {}
        self.req = urllib2.Request(
            proto + '://' + quest_url + page +
            ('?' + urllib.urlencode(query) if query != {} else ''))

    def add_header(self, key, val):
        """Add key and val to the header of self.req."""

        self.req.add_header(key, val)

    def add_form(self, key, val):
        """Add key=val to the data field of self.req."""

        if not isinstance(self.data, dict):
            raise Error(
                "Need to call self.make_request before calling self.add_form.")
        if not isinstance(key, str):
            key = str(key)
        if not isinstance(val, str):
            val = str(val)
        self.data[key] = val

    def add_forms(self, d):
        """Add key, value pairs from d to self.data."""

        for k, v in d.items():
            self.add_form(k, v)

    def login(self, password):
        """Login to QUEST."""

        assert password is not None

        self.make_request('/psc/AS/', {'cmd': 'login',
                                       'languageCd': 'ENG'})
        self.add_forms({'userid': self.questid,
                        'pwd': password,
                        'httpPort': '',
                        'timezoneOffset': '240',
                        'Submit': 'Sign+in'})
        self.get_page()

        if self.page.find(text=invalid_id_pass_msg):
            raise BadLogin(invalid_id_pass_msg)


proto = 'https'
quest_url = 'quest.pecs.uwaterloo.ca'
invalid_id_pass_msg = 'Your User ID and/or Password are invalid.'



# Previewing HTML code in browser.

# Copyright (C) 2015 Kieran Colford
#
# This file is part of UWaterloo-AddCourse.
#
# UWaterloo-AddCourse is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# UWaterloo-AddCourse is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with UWaterloo-AddCourse.  If not, see
# <http://www.gnu.org/licenses/>.


import tempfile
import time
import webbrowser


def preview_page(html):
    """Preview the HTML code html in your browser."""

    with tempfile.NamedTemporaryFile(mode='w+t') as f:
        f.write(str(html))
        f.flush()
        webbrowser.open('file://' + f.name)
        time.sleep(5)




# Error module.

# Copyright (C) 2015 Kieran Colford
#
# This file is part of UWaterloo-AddCourse.
#
# UWaterloo-AddCourse is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# UWaterloo-AddCourse is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with UWaterloo-AddCourse.  If not, see
# <http://www.gnu.org/licenses/>.


"""Classes for errors."""


class Error(Exception):
    """An error thrown from the addcourse package."""

    pass


class BadLogin(Error):
    """Incorrect username or password."""


    pass




"""Store package metadata."""

__author__ = 'Kieran Colford'
__license__ = 'GPLv3+'
__copyright__ = 'Copyright (C) 2015 Kieran Colford'
__credits__ = """Mark Petrick, for the support that inspired this project.
"""
__version__ = '1.0.1'



# Add the course we want.

# Copyright (C) 2015 Kieran Colford
#
# This file is part of UWaterloo-AddCourse.
#
# UWaterloo-AddCourse is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# UWaterloo-AddCourse is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with UWaterloo-AddCourse.  If not, see
# <http://www.gnu.org/licenses/>.


"""Class to add the courses we want."""


import itertools
from .class_numbers import numbers
from .addpage import AddPage
from .preview import preview_page


class AddCourse(AddPage):
    """Add a course."""

    def register_class(self, classno, tutno=0):
        """Register into the class classno."""

        self.do_action('DERIVED_REGFRM1_SSR_PB_ADDTOLIST2$9$',
                       {'DERIVED_REGFRM1_CLASS_NBR': classno})

        if not self.page.find(text=has_class_message):
            self.register_tutorial(tutno)

    def register_tutorial(self, tutno=0):
        """Register into tutorial tutno."""
        
        if tutno != 0:
            if not isinstance(tutno, str):
                tutno = str(tutno)
            node = self.page.find(text=tutno)
            nodekey = node.parent['id'].split('$')[-1]
        else:
            node = self.page.find_all(name='img', attrs={'alt': 'Open'})[1]
            nodekey = node.parent.parent['id'].split('$')[-1]
        self.do_action('DERIVED_CLS_DTL_NEXT_PB',
                       {'SSR_CLS_TBL_R1$sels$0': nodekey})

    def get_registered_class(self, display_page_on_fail=False):
        """Try to get into the class we've registered into."""

        self.do_action('DERIVED_CLS_DTL_NEXT_PB$280$',
                       {'DERIVED_CLS_DTL_CLASS_PRMSN_NBR$118$': ""})

        self.do_action("DERIVED_REGFRM1_LINK_ADD_ENRL$82$")

        self.do_action("DERIVED_REGFRM1_SSR_PB_SUBMIT")

        if len(self.page.find_all(name='img', attrs={'alt': 'Error'})) >= 2:
            if display_page_on_fail:
                print self.page
            self.do_action('DERIVED_REGFRM1_SSR_LINK_STARTOVER')
            return False
        else:
            return True

    def get_class(self, classno, tutno=0, display_page_on_fail=False):
        """Attempt to register into a class with class number classno."""

        self.get_add()
        self.register_class(classno, tutno)
        return self.get_registered_class(display_page_on_fail)

    def clear_class(self):
        """Clear the class we have saved."""

        self.do_action('P_DELETE$0')

    def run(self, course):
        """Keep requesting the class we want from QUEST."""

        if isinstance(course, str):
            class_numbers = numbers(course)
        elif isinstance(course, list):
            class_numbers = [int(n) for n in course]
        else:
            raise TypeError("course is not a list or str")

        self.get_add()
        "Navigated to Add page, begining reqests."

        for i in itertools.count():
            for cls in class_numbers:
                self.register_class(cls)
                try:
                    for j in range(repeat):
                        if self.get_registered_class():
                            print "Attempt", i * repeat + j + 1, "on class", cls, "Succeeded."
                            preview_page(self.page)
                            return
                        else:
                            print "Attempt", i * repeat + j + 1, "on class", cls, "Failed."
                finally:
                    self.clear_class()


def addcourse(user, password, course):
    """Run the QuestBrowser and get into the course we need."""

    try:
        AddCourse(user, password).run(course)
    except Error as e:
        print >> sys.stderr, e
        raw_input("Press Enter to continue...")
        exit(1)


has_class_message = 'This class is already in your Shopping Cart.  Try another.'
repeat = 2



#!/usr/bin/env python

"""Package for easily adding a course on UWaterloo's QUEST."""

from setuptools import *
from addcourse.version import __version__

with open('README.rst') as f:
    long_description = f.read()

setup(
    name='uwaterloo-addcourse',
    version=__version__,
    author='Kieran Colford',
    author_email='colfordk@gmail.com',
    url='https://github.com/kcolford/uwaterloo-addcourse',
    license='GPLv3+',
    description=__doc__,
    long_description=long_description,
    keywords=[
        'uwaterloo',
        'addcourse',
    ],
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'addcourse = addcourse:main',
        ],
    },
    install_requires=[
        'beautifulsoup4>=4.2.0',
    ],
    use_2to3=True,
)
